import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JButton;


/**
 * @author Ben Coleman
 *
 * Sets up the container to display examples of Brownian Motion
 */
@SuppressWarnings("serial")
public class BrownianPresets extends JPanel implements ActionListener
{
	private final double CONTAINER_RADIUS = 35 / Math.pow(10, 10);
	private final double BOLTZMANN_CONSTANT = 13806.50524 / Math.pow(10, 27);
	private final double DISTANCE_SCALE = Math.pow(10, -10);
	
	private final int OPTION_ONE_MASS_SCLAE_FACTOR = 35;
	private final int OPTION_ONE_RADIUS_SCALE_FACTOR = 9;
	private final long SEED_OPTION_ONE = 1;
	private final int NUM_OBJECTS_OPTION_ONE = 101;
	private final double TEMPERATURE_OPTION_ONE = 50;
	private final int SELECTED_OBJECT_INDEX_OPTION_ONE = 72;

	private final int OPTION_TWO_MASS_SCLAE_FACTOR = 30;
	private final int OPTION_TWO_RADIUS_SCALE_FACTOR = 7;
	private final long SEED_OPTION_TWO = 9;
	private final int NUM_OBJECTS_OPTION_TWO = 102;
	private final double TEMPERATURE_OPTION_TWO = 50;
	private final int SELECTED_OBJECT_INDEX_OPTION_TWO_1 = 71;
	private final int SELECTED_OBJECT_INDEX_OPTION_TWO_2 = 44;

	private final int OPTION_THREE_MASS_SCLAE_FACTOR = 50;
	private final int OPTION_THREE_RADIUS_SCALE_FACTOR = 7;
	private final long SEED_OPTION_THREE = 2;
	private final int NUM_OBJECTS_OPTION_THREE = 31;
	private final double TEMPERATURE_OPTION_THREE = 50;
	private final int SELECTED_OBJECT_INDEX_OPTION_THREE = 26;
	private final double CONTAINER_RADIUS_MODIFIER_OPTION_THREE = 4 / 7.0;

	private final int OPTION_FOUR_MASS_SCLAE_FACTOR = 55;
	private final int OPTION_FOUR_RADIUS_SCALE_FACTOR = 16;
	private final long SEED_OPTION_FOUR = 5;
	private final int NUM_OBJECTS_OPTION_FOUR = 53;
	private final double TEMPERATURE_OPTION_FOUR = 150;
	private final int SELECTED_OBJECT_INDEX_OPTION_FOUR_1 = 51;
	private final int SELECTED_OBJECT_INDEX_OPTION_FOUR_2 = 39;
	private final int SELECTED_OBJECT_INDEX_OPTION_FOUR_3 = 10;
	private final double CONTAINER_RADIUS_MODIFIER_OPTION_FOUR = 2;

	private final int OPTION_FIVE_MASS_SCLAE_FACTOR = 25;
	private final int OPTION_FIVE_RADIUS_SCALE_FACTOR = 7;
	private final long SEED_OPTION_FIVE = 1;
	private final int NUM_OBJECTS_OPTION_FIVE = 51;
	private final double TEMPERATURE_OPTION_FIVE = 50;
	private final int SELECTED_OBJECT_INDEX_OPTION_FIVE = 14;
	private final double CONTAINER_RADIUS_MODIFIER_OPTION_FIVE = 5 / 7.0;
	
	private Simulation mySim;

	private JButton brownian1;
	private JButton brownian2;
	private JButton brownian3;
	private JButton brownian4;
	private JButton brownian5;
	private JButton cancel;
	
	private ControlPanel control;
	
	private boolean cancelLast;
	
	public BrownianPresets(Simulation sim, ControlPanel cPanel)
	{
		control = cPanel;
		mySim = sim;
		cancelLast = false;

		brownian1 = new JButton("1 35 AMU ball with 100 1 AMU atoms");
		brownian1.setActionCommand("one");
		brownian1.addActionListener(this);
		
		brownian2 = new JButton("2 30 AMU balls with 100 1 AMU atoms");
		brownian2.setActionCommand("two");
		brownian2.addActionListener(this);
		
		brownian3 = new JButton("1 50 AMU ball with 30 1 AMU atoms");
		brownian3.setActionCommand("three");
		brownian3.addActionListener(this);
		
		brownian4 = new JButton("3 55 AMU balls with 50 1 AMU atoms");
		brownian4.setActionCommand("four");
		brownian4.addActionListener(this);
		
		brownian5 = new JButton("1 25 AMU ball with 50 1 AMU atoms");
		brownian5.setActionCommand("five");
		brownian5.addActionListener(this);
		
		cancel = new JButton("Cancel");
		cancel.setActionCommand("Cancel");
		cancel.addActionListener(this);
		
		JPanel panel = new JPanel(new GridLayout(0, 1));
		panel.add(brownian1);
		panel.add(brownian2);
		panel.add(brownian3);
		panel.add(brownian4);
		panel.add(brownian5);
		panel.add(cancel);
		add(panel, BorderLayout.CENTER);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		cancelLast = false;
		
		if(e.getActionCommand().equals("one"))
		{
			mySim.clearSim();
			setUpOne();
		}
		
		if(e.getActionCommand().equals("two"))
		{
			mySim.clearSim();
			setUpTwo();
		}
		
		if(e.getActionCommand().equals("three"))
		{
			mySim.clearSim();
			setUpThree();
		}
		
		if(e.getActionCommand().equals("four"))
		{
			mySim.clearSim();
			setUpFour();
		}
		
		if(e.getActionCommand().equals("five"))
		{
			mySim.clearSim();
			setUpFive();
		}
		
		if(e.getActionCommand().equals("Cancel"))
		{
			cancelLast = true;
		}
		
		double kineticEnergy = 0;
		
		for(Atom obj2: mySim.getObjects())
		{
			kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
		}
		
		kineticEnergy /= mySim.getObjects().size();
		mySim.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);
		
		control.hasSelected(cancelLast);
	}

	private void setUpFive() 
	{
		mySim.changeNumObjects(NUM_OBJECTS_OPTION_FIVE);
		mySim.setContainerRadius(CONTAINER_RADIUS * CONTAINER_RADIUS_MODIFIER_OPTION_FIVE);
		mySim.setTemperature(TEMPERATURE_OPTION_FIVE);
		mySim.setSeed(SEED_OPTION_FIVE);
		mySim.reset();
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FIVE).setRadius(OPTION_FIVE_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FIVE).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FIVE).getMass() * OPTION_FIVE_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FIVE), Main.totalTime);
	}

	private void setUpFour() 
	{
		mySim.changeNumObjects(NUM_OBJECTS_OPTION_FOUR);
		mySim.setContainerRadius(CONTAINER_RADIUS * CONTAINER_RADIUS_MODIFIER_OPTION_FOUR);
		mySim.setTemperature(TEMPERATURE_OPTION_FOUR);
		mySim.setSeed(SEED_OPTION_FOUR);
		mySim.reset();
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_1).setRadius(OPTION_FOUR_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_1).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_1).getMass() * OPTION_FOUR_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_1), Main.totalTime);
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_2).setRadius(OPTION_FOUR_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_2).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_2).getMass() * OPTION_FOUR_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_2), Main.totalTime);
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_3).setRadius(OPTION_FOUR_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_3).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_3).getMass() * OPTION_FOUR_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_FOUR_3), Main.totalTime);
	}

	private void setUpThree() 
	{
		mySim.changeNumObjects(NUM_OBJECTS_OPTION_THREE);
		mySim.setContainerRadius(CONTAINER_RADIUS * CONTAINER_RADIUS_MODIFIER_OPTION_THREE);
		mySim.setTemperature(TEMPERATURE_OPTION_THREE);
		mySim.setSeed(SEED_OPTION_THREE);
		mySim.reset();
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_THREE).setRadius(OPTION_THREE_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_THREE).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_THREE).getMass() * OPTION_THREE_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_THREE), Main.totalTime);
	}

	private void setUpTwo() 
	{
		mySim.changeNumObjects(NUM_OBJECTS_OPTION_TWO);
		mySim.setContainerRadius(CONTAINER_RADIUS);
		mySim.setTemperature(TEMPERATURE_OPTION_TWO);
		mySim.setSeed(SEED_OPTION_TWO);
		mySim.reset();
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_1).setRadius(OPTION_TWO_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_1).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_1).getMass() * OPTION_TWO_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_1), Main.totalTime);
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_2).setRadius(OPTION_TWO_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_2).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_2).getMass() * OPTION_TWO_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_TWO_2), Main.totalTime);
	}

	private void setUpOne() 
	{
		mySim.changeNumObjects(NUM_OBJECTS_OPTION_ONE);
		mySim.setContainerRadius(CONTAINER_RADIUS);
		mySim.setTemperature(TEMPERATURE_OPTION_ONE);
		mySim.setSeed(SEED_OPTION_ONE);
		mySim.reset();
		
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_ONE).setRadius(OPTION_ONE_RADIUS_SCALE_FACTOR * DISTANCE_SCALE);
		mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_ONE).setMass(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_ONE).getMass() * OPTION_ONE_MASS_SCLAE_FACTOR);
		mySim.updateCollisions(mySim.getObjects().get(SELECTED_OBJECT_INDEX_OPTION_ONE), Main.totalTime);
		
	}
}
