
/**
 * @author Ben Coleman
 *
 * This is a stores the data for a future collision event between two Atoms.  It is ordered by time.
 * The first Atom is the atom that this collision was calculated for and the second is the Atom
 * (or container if the second Atom is null) that the first is colliding with.
 * The time is the time at which this event will occur.
 */
public class CollisionEvent implements Comparable <CollisionEvent> {

	private Atom atomOne;
	private Atom atomTwo;
	private double timeToCollision;
	private int numHits;
	
	public CollisionEvent(Atom one, Atom two, double time, int hits)
	{
		atomOne = one;
		atomTwo = two;
		timeToCollision = time;
		numHits = hits;
	}
	
	@Override
	public int compareTo(CollisionEvent arg0) {
		// TODO Auto-generated method stub
		double difference = timeToCollision - arg0.timeToCollision;
		if(difference == 0)
		{
			return 0;
		}
		if(difference < 0 && timeToCollision != -1 || arg0.timeToCollision == -1)
		{
			return -1;
		}
		else
		{
			return 1;
		}
	}
	
	public double getTime()
	{
		return timeToCollision;
	}
	
	public Atom getAtomOne()
	{
		return atomOne;
	}
	
	public Atom getAtomTwo()
	{
		return atomTwo;
	}
	
	public String toString()
	{
		int i = -1;
		
		if(atomTwo != null)
		{
			i = atomTwo.getId();
		}
		
		return atomOne.getId() + ", " + i + ", " + timeToCollision;
	}
	
	public boolean equals(CollisionEvent e)
	{
		if(e == null)
		{
			return false;
		}
		return atomTwo == e.atomTwo && atomOne == e.atomOne || atomOne == e.atomTwo && atomTwo == e.atomOne;
	}
	
	public int getHits()
	{
		return numHits;
	}

}
