/**
 * @author original c++ code: Ng Weng Leong, Java port + graphics: Ben Coleman
 * Provides the main method along with a static input method that can be used when inputs must be numbers
 *
 */
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JWindow;
import javax.swing.filechooser.FileNameExtensionFilter;

import java.util.regex.Pattern;

import java.io.File;

public class Main 
{
	
	private static final double HYDROGEN_RADIUS = 5.29e-11;
	private static final double HYDROGEN_MASS = 1.6736 / Math.pow(10, 27);
	private static final double BOLTZMANN_CONSTANT = 13806.50524 / Math.pow(10, 27);
	private static final int DEFAULT_NUM_ATOMS = 100;
	private static final double DEFAULT_CONTAINER_RADIUS = 35;
	private static final double DEFAULT_TEMPERATURE = 50;
	private static final long DEFAULT_SEED = 1;
	private static final int DISPLAY_FRAME_X = 300 * 2 + 8;
	private static final int DISPLAY_FRAME_Y = 300 * 2 + 34 + 
											   SimulationGraphicsPanel.TEXT_AREA_LINE_HEIGHT * 
											   SimulationGraphicsPanel.TEXT_AREA_LINES +
											   SimulationGraphicsPanel.TEXT_AREA_BOTTOM_BORDER;
	private static final int CONTROL_FRAME_X = 225;
	private static final int CONTROL_FRAME_Y = 495;
	private static final int DISPLAY_FRAME_START_X = 0;
	private static final int DISPLAY_FRAME_START_Y = 0;
	private static final int CONTROL_FRAME_START_X = 300 * 2 + 8;
	private static final int CONTROL_FRAME_START_Y = 0;
	private static final int LOAD_FRAME_Y_SIZE = 100;
	private static final int LOAD_FRAME_X_SIZE = 310;
	private static final int LOAD_FRAME_Y_LOC = 267;
	private static final int LOAD_FRAME_X_LOC = 358;
	private static final int TENTH_SECOND = 100;
	private static final int ONE_NANOSECOND = 1;
	
	public static double totalTime;

	//print usage not implemented as it should not be needed due to the modified input scheme
	
	public static void main(String[] arg)
	{	
		//shows a dialog that lets the user choose the initial number of atoms
		int n = DEFAULT_NUM_ATOMS;
		String str = Main.guardedInput("Number of Atoms (> 0)" + "\n" +
									   "Note: Running the simulation with more than 5000 atoms can cause severe performance problems",
									   n + "", true);
		
		if(str != null && !str.equals(""))
		{
			n = (int)Double.parseDouble(str);
		}
		
		while(n <= 0)
		{
			JOptionPane.showMessageDialog(null, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
			n = 100;
			str =  Main.guardedInput("Number of Atoms (> 0)" + "\n" +
									 "Note: Running the simulation with more than 5000 atoms can cause severe performance problems",
									 n + "", true);
			
			if(str != null && !str.equals(""))
			{
				n = (int)Double.parseDouble(str);
			}
		}
		
		//shows a dialog that lets the user choose the initial container radius
		double r = DEFAULT_CONTAINER_RADIUS;
		str =  Main.guardedInput("Container Radius (Angstroms > 0)", r + "", true);
		
		if(str != null && !str.equals(""))
		{
			r = Double.parseDouble(str);
		}
		
		while(r <= 0 || n > Main.calculateNumCells(r / Math.pow(10, 10)))
		{
			if(r <= 0)
			{
				JOptionPane.showMessageDialog(null, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
			}
			
			else
			{
				JOptionPane.showMessageDialog(null, "Error: Please enter a value that is large enought accomodate all of the atoms", null, JOptionPane.ERROR_MESSAGE);
			}
			
			r = 35;
			str =  Main.guardedInput("Container Radius (Angstroms > 0)", r + "", true);
			
			if(str != null && !str.equals(""))
			{
				r = Double.parseDouble(str);
			}
		}
		
		//shows a dialog that lets the user choose the initial temperature
		double t = DEFAULT_TEMPERATURE;
		str =  Main.guardedInput("Temperature (Kelvins > 0)" + "\n" +
								 "Note: Extremely high temperatures lead to choppy motion",
								 t + "", true);
		
		if(str != null && !str.equals(""))
		{
			t = Double.parseDouble(str);
		}
		
		while(t <= 0)
		{
		 	JOptionPane.showMessageDialog(null, "Error: Please enter a value greater than 0", null, JOptionPane.ERROR_MESSAGE);
		 	t = 50;
		 	str =  Main.guardedInput("Temperature (Kelvins > 0)" + "\n" +
		 							 "Note: Extremely high temperatures lead to choppy motion",
		 							 t + "", true);
		
			if(str != null && !str.equals(""))
			{
				t = Double.parseDouble(str);
			}
			
		}
		
		//shows a dialog that lets the user choose the initial seed
		long seed = DEFAULT_SEED;
		str =  Main.guardedInput("Seed", seed + "", true);
		
		if(str != null && !str.equals(""))
		{
			seed = (long)Double.parseDouble(str);
		}
		
		//shows a dialog that lets the user choose whether a log file is used, 
		//and if one is used, what it shall be named
		JFileChooser chooser = new JFileChooser();
	    FileNameExtensionFilter filter = new FileNameExtensionFilter("TXT files", "txt");
	    chooser.setFileFilter(filter);
	    chooser.setDialogTitle("Please save a log file, enter \"null\" for no log file");
	    int returnVal = chooser.showSaveDialog(new JFrame());
	    
	    if(returnVal == JFileChooser.APPROVE_OPTION) 
	    {
	    	if(!chooser.getSelectedFile().getName().equals("null"))
	    	{
	    		System.out.println(chooser.getCurrentDirectory().getName() + chooser.getSelectedFile().getName() + ".txt");
	    		str = chooser.getSelectedFile().getName();
	    		
	    		if(str.length() > 4 && !str.substring(str.length() - 4).equals(".txt"))
	    		{
	    			str = str + ".txt";
	    		}
	    		
	    		LogUtility.setFile(new File(chooser.getCurrentDirectory(), str));
	    		LogUtility.startLogging();
	    	}
	       
	    }
	    
	    else
	    {
	    	System.exit(0);
	    }
		
		//creates the splash that lets the user know the program is loading
		JWindow loader = new JWindow();						
		loader.setSize(LOAD_FRAME_X_SIZE, LOAD_FRAME_Y_SIZE);
		loader.setLocation(LOAD_FRAME_X_LOC, LOAD_FRAME_Y_LOC);
		loader.setContentPane(new LoadScreen());
		loader.repaint();
		loader.setVisible(true);
		
		Simulation sim = new Simulation(n, HYDROGEN_RADIUS, HYDROGEN_MASS, r / Math.pow(10, 10), t, seed, null);
		printStartStats(sim);
		
		totalTime = 0;
		
		//creates the frames that hold the gui for the simulation
		//both are set to hide when they are closed so that the statistics can 
		//be printed to the log file
		JFrame frame = new JFrame("Gas Simulation");
		JFrame controlFrame = new JFrame("Control");
		
		frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		frame.setSize(DISPLAY_FRAME_X, DISPLAY_FRAME_Y);
		frame.setLocation(DISPLAY_FRAME_START_X, DISPLAY_FRAME_START_Y);
		
		ControlPanel cPanel = new ControlPanel(sim);
		sim.updateControlPanel(cPanel);
		
		SimulationGraphicsPanel panel = new SimulationGraphicsPanel(sim, cPanel);
		frame.setContentPane(panel);
		panel.repaint();
		
		controlFrame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		controlFrame.setSize(CONTROL_FRAME_X, CONTROL_FRAME_Y);
		controlFrame.setLocation(CONTROL_FRAME_START_X, CONTROL_FRAME_START_Y);
		controlFrame.setContentPane(cPanel);
		
		controlFrame.setVisible(true);
		
		frame.setVisible(true);
		
		loader.setVisible(false);
		
		while(frame.isVisible() && controlFrame.isVisible())
		{
			while(cPanel.isRunning())
			{
				totalTime = sim.advance(totalTime);
				panel.repaint();
				
				try
				{
					Thread.sleep(cPanel.getSpeed(), ONE_NANOSECOND);
				}
				
				catch(InterruptedException e)
				{
					System.exit(0);
				}
				
				if(!controlFrame.isVisible() || !frame.isVisible())
				{
					frame.setVisible(false);
					cPanel.stopRunning();
				}
				
			}
			
			//resets the simulation completely
			if(cPanel.reset())
			{
				sim.printStats(totalTime);
				totalTime = 0;
				sim.reset();
				SimulationGraphicsPanel.enableContainerRepaint();
				panel.repaint();
				cPanel.setReset(false);
				printStartStats(sim);
			}
			
			if(cPanel.needsRepaint())
			{
				panel.repaint();
				cPanel.toggleRepaintRequest();
			}
				
			//keep cpu usage low
			try 
			{
				Thread.sleep(TENTH_SECOND);
			} 
				
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
			
		}
		
		sim.printStats(totalTime);
		
		LogUtility.stopLogging();
		LogUtility.closeFile();
		System.exit(0);
		
	}
	
	//provides an input that will ensure that the number returned is a double
	//the String is the message that is to be displayed on the input dialog
	//the Object is what is initially displayed in the input field
	//the boolean is to determine whether the input it one of the starting inputs or not
	//this is to make it possible to exit the simulation without putting in all of the inputs
	//and waiting for it to load
	public static String guardedInput(String str, Object obj, boolean isInitialInput)
	{
		String s = JOptionPane.showInputDialog(str, obj);
		
		if(s == null)
		{
			if(isInitialInput)
			{
				System.exit(0);
			}
			
			else
			{
				return "";
			}
			
		}
		
		while(!isNumber(s))
		{
			JOptionPane.showMessageDialog(null, "Error: the input was not valid\n" +
												"A valid input is a number that can have a sign and/or and exponent\n" +
												"In addition, it can't end in a decimal and the exponent can't contain a decimal");
			s = JOptionPane.showInputDialog(str);
		}
		
		return s;
	}
	
	//determines if the string can be parsed
	//this might require updating if a wider array of formats need to be allowed
	public static boolean isNumber(String str)
	{
		char [] chars = str.toCharArray();
		int exponentFound = -1;
		int decimalFound = -1;
		for(int i = 0; i < chars.length; i ++)
		{
			char ch = chars[i];
			if('.' == ch)
			{
				if(decimalFound != -1 || i + 1 == chars.length || exponentFound != -1)
				{
					return false;
				}
				
				decimalFound = i;
			}
			else if('e' == ch || 'E' == ch)
			{
				if(decimalFound == i - 1 || i + 1 == chars.length || exponentFound != -1)
				{
					return false;
				}
				
				exponentFound = i;
			}
			else if('-' == ch || '+' == ch)
			{
				if(i != 0 && exponentFound != i - 1)
				{
					return false;
				}
			}
			else if(!Pattern.matches("[1234567890]", "" + ch))
			{
				return false;
			}
		}
		return true;
	}
	
	//provides the starting statistics for the simulation
	//displayed at the start and whenever the simulation is reset
	public static void printStartStats(Simulation sim)
	{
		LogUtility.printToLogFile("");
		LogUtility.printToLogFile("System Parameters");
		LogUtility.printToLogFile("-----------------");
		LogUtility.printToLogFile("Number of atoms = " + sim.getNumAtoms());
		LogUtility.printToLogFile("Atomic radius (hydrogen at the ground state) = " + HYDROGEN_RADIUS + " m");
		LogUtility.printToLogFile("Atomic mass of hydrogen = " +  HYDROGEN_MASS + " kg");
		LogUtility.printToLogFile("Container Radius = " + sim.getContainerRadius() + " m");
		LogUtility.printToLogFile("Temperature (T) = " + sim.getTemperature() + " K");
		double volume = StrictMath.PI * 
						(StrictMath.pow(sim.getContainerRadius(), 2) - StrictMath.pow(HYDROGEN_RADIUS, 2) * sim.getNumAtoms());
		LogUtility.printToLogFile("Unoccupied volume in container (V - nv) = " + volume + " m^2");
		LogUtility.printToLogFile("Experimental Boltzmann Constant  = " + (BOLTZMANN_CONSTANT)
								  + " kg(m^2)/(K(s^2))");
		LogUtility.printToLogFile("Seed = " + sim.getSeed());
	}
	
	//used to determine whether a given container radius can hold all of the atoms
	//returns the number of cells for a container of the given radius
	public static int calculateNumCells(double radius)
	{
		int length = (int)((radius - HYDROGEN_RADIUS) / (2 * HYDROGEN_RADIUS)) + 1;
		int sum = 0;
		
		for(int i = 0; i < length; i ++)
		{
			double inc = (radius - HYDROGEN_RADIUS) / (length - 1);
			double ccr = i * inc;
			int nCells_cc = (int)(StrictMath.PI / StrictMath.atan(HYDROGEN_RADIUS / ccr)) - 1;
			sum += nCells_cc;
		}

		return sum;
	}
	
}
