import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JButton;


/**
 * @author Ben Coleman
 * provides examples of one-dimensional collisions
 * button labeling syntax: (one line)
 * (<mass of left atom>, <speed of left atom>)direction of left atom, 
 * (<mass of right atom>, <speed of right atom>)direction of right atom
 *
 */
@SuppressWarnings("serial")
public class OneDimensionalPresets extends JPanel implements ActionListener 
{
	private final double HYDROGEN_RADIUS = 5.29e-11;
	private final double HYDROGEN_MASS = 1.6736 / Math.pow(10, 27);
	private final double SPEED = 500;
	private final double CONTAINER_RADIUS = 15 / Math.pow(10, 10);
	private final double BOLTZMANN_CONSTANT = 13806.50524 / Math.pow(10, 27);
	private final double START_POSITION = 12e-10;
	private final double MIN_SPEED = 1e-100;
	
	private Simulation mySim;

	private JButton button1;
	private JButton button2;
	private JButton button3;
	private JButton button4;
	private JButton button5;
	private JButton button6;
	private JButton button7;
	private JButton button8;
	private JButton button9;
	private JButton button10;
	private JButton button11;
	private JButton cancel;
	
	private ControlPanel control;
	
	private boolean cancelLast;
	
	public OneDimensionalPresets(Simulation sim, ControlPanel cPanel)
	{
		control = cPanel;
		mySim = sim;
		cancelLast = false;
		
		button1 = new JButton("(m,v)>, (m,0)");
		button1.setActionCommand("one");
		button1.addActionListener(this);

		button2 = new JButton("(m,v)>, (2m,0)");
		button2.setActionCommand("two");
		button2.addActionListener(this);

		button3 = new JButton("(2m,v)>, (m,0)");
		button3.setActionCommand("three");
		button3.addActionListener(this);

		button4 = new JButton("(m,v)>, (m,v)<");
		button4.setActionCommand("four");
		button4.addActionListener(this);

		button5 = new JButton("(2m,v)>, (m,v)<");
		button5.setActionCommand("five");
		button5.addActionListener(this);

		button6 = new JButton("(m,2v)>, (m,v)<");
		button6.setActionCommand("six");
		button6.addActionListener(this);

		button7 = new JButton("(2m,2v)>, (m,v)<");
		button7.setActionCommand("seven");
		button7.addActionListener(this);

		button8 = new JButton("(m,2v)>, (2m,v)<");
		button8.setActionCommand("eight");
		button8.addActionListener(this);

		button9 = new JButton("(m,2v)>, (m,v)>");
		button9.setActionCommand("nine");
		button9.addActionListener(this);

		button10 = new JButton("(2m,2v)>, (m,v)>");
		button10.setActionCommand("ten");
		button10.addActionListener(this);
		
		button11 = new JButton("(m,2v)>, (2m,v)>");
		button11.setActionCommand("eleven");
		button11.addActionListener(this);
		
		cancel = new JButton("Cancel");
		cancel.setActionCommand("Cancel");
		cancel.addActionListener(this);

		JPanel subPanel1 = new JPanel(new GridLayout(1, 0));
		subPanel1.add(button1);
		subPanel1.add(button4);
		subPanel1.add(button9);
		
		JPanel subPanel2 = new JPanel(new GridLayout(1, 0));
		subPanel2.add(button2);
		subPanel2.add(button5);
		subPanel2.add(button10);
		
		JPanel subPanel3 = new JPanel(new GridLayout(1, 0));
		subPanel3.add(button3);
		subPanel3.add(button6);
		subPanel3.add(button11);
		
		JPanel subPanel4 = new JPanel(new GridLayout(1, 0));
		subPanel4.add(new JPanel());
		subPanel4.add(button7);
		subPanel4.add(new JPanel());
		
		JPanel subPanel5 = new JPanel(new GridLayout(1, 0));
		subPanel5.add(new JPanel());
		subPanel5.add(button8);
		subPanel5.add(new JPanel());
		
		JPanel panel = new JPanel(new GridLayout(0,1));
		panel.add(subPanel1);
		panel.add(subPanel2);
		panel.add(subPanel3);
		panel.add(subPanel4);
		panel.add(subPanel5);
		panel.add(cancel);
		
		/*Box cancelPanel = new Box(BoxLayout.X_AXIS);
		cancelPanel.add(Box.createHorizontalGlue());
		cancelPanel.add(cancel);
		cancelPanel.add(Box.createHorizontalGlue());
		
		Box panel2 = new Box(BoxLayout.Y_AXIS);
		panel2.add(panel);
		panel2.add(cancelPanel, BorderLayout.CENTER);*/
		
		add(panel, BorderLayout.CENTER);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		cancelLast = false;
		
		if(e.getActionCommand().equals("one"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpOne();
		}
		
		if(e.getActionCommand().equals("two"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpTwo();
		}
		
		if(e.getActionCommand().equals("three"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpThree();
		}
		
		if(e.getActionCommand().equals("four"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpFour();
		}
		
		if(e.getActionCommand().equals("five"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpFive();
		}
		
		if(e.getActionCommand().equals("six"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpSix();
		}
		
		if(e.getActionCommand().equals("seven"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpSeven();
		}
		
		if(e.getActionCommand().equals("eight"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpEight();
		}
		
		if(e.getActionCommand().equals("nine"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpNine();
		}
		
		if(e.getActionCommand().equals("ten"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpTen();
		}
		
		if(e.getActionCommand().equals("eleven"))
		{
			mySim.setContainerRadius(CONTAINER_RADIUS);
			mySim.clearSim();
			setUpEleven();
		}
		
		if(e.getActionCommand().equals("Cancel"))
		{
			cancelLast = true;
		}
		
		double kineticEnergy = 0;
		
		for(Atom obj2: mySim.getObjects())
		{
			kineticEnergy += StrictMath.pow(obj2.getLinearSpeed(), 2) * obj2.getMass() / 2;
		}
		
		kineticEnergy /= mySim.getObjects().size();
		mySim.setTemperature(kineticEnergy / BOLTZMANN_CONSTANT);

		control.hasSelected(cancelLast);
	}

	private void setUpEleven() 
	{
		Atom one = new Atom(new Vec(0, SPEED), new Vec(0, 0), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION / 2), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpTen()
	{
		Atom one = new Atom(new Vec(0, SPEED), new Vec(0, 0), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION / 2), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpNine() 
	{
		Atom one = new Atom(new Vec(0, SPEED), new Vec(0, 0), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION / 2), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpEight()
	{
		Atom one = new Atom(new Vec(0, -SPEED), new Vec(0, START_POSITION / 2), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpSeven() 
	{
		Atom one = new Atom(new Vec(0, -SPEED), new Vec(0, START_POSITION / 2), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 1);
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpSix() 
	{
		Atom one = new Atom(new Vec(0, -SPEED), new Vec(0, START_POSITION / 2), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED * 2), new Vec(0, -START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpFive() 
	{
		Atom one = new Atom(new Vec(0, -SPEED), new Vec(0, START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED), new Vec(0, -START_POSITION), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 1);
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

	private void setUpFour() 
	{
		Atom one = new Atom(new Vec(0, -SPEED), new Vec(0, START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED), new Vec(0, -START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
	}

	private void setUpThree() 
	{
		Atom one = new Atom(new Vec(0, MIN_SPEED), new Vec(0, 0), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED), new Vec(0, -START_POSITION), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
	}

	private void setUpTwo()
	{
		Atom one = new Atom(new Vec(0, MIN_SPEED), new Vec(0, 0), HYDROGEN_MASS * 2, HYDROGEN_RADIUS * 2, 0);
		Atom two = new Atom(new Vec(0, SPEED), new Vec(0, -START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
	}

	private void setUpOne() 
	{
		Atom one = new Atom(new Vec(0, MIN_SPEED), new Vec(0, 0), HYDROGEN_MASS, HYDROGEN_RADIUS, 0);
		Atom two = new Atom(new Vec(0, SPEED), new Vec(0, -START_POSITION), HYDROGEN_MASS, HYDROGEN_RADIUS, 1);
		mySim.getObjects().clear();
		mySim.clearCollisions();
		mySim.getObjects().add(one);
		mySim.getObjects().add(two);
		mySim.updateCollisions(one, Main.totalTime);
		mySim.updateCollisions(two, Main.totalTime);
	}

}
