/**
 * @author original c++ code: Ng Weng Leong, Java port: Ben Coleman
 * 2D vector  
 * Provides basic operations, as well as operations for obtaining unit vectors 
 * and orthogonal vectors
 * Note: the constructor is (y, x), not (x, y)
 *
 */
public class Vec 
{
	
	private double myX;
	private double myY;
	
	public Vec(double y, double x)
	{
		myX = x;
		myY = y;
	}
	
	public Vec add(Vec v)
	{
		return new Vec(myY + v.myY, myX + v.myX);
	}
	
	public Vec subtract(Vec v)
	{
		return new Vec(myY - v.myY, myX - v.myX);
	}
	
	public Vec scalarMultiplication(double scalar)
	{
		return new Vec(myY * scalar, myX * scalar);
	}
	
	public Vec scalarDivision(double scalar)
	{
		return new Vec(myY / scalar, myX / scalar);
	}
	
	public Vec scalarDivisionEquals(double scalar)
	{
		myX /= scalar;
		myY /= scalar;
		return new Vec(myY, myX);
	}
	
	public double dotProduct(Vec v)
	{
		return (myY * v.myY) + (myX * v.myX);
	}
	
	public Vec unitVector()
	{
		double magnitude = StrictMath.sqrt(myX * myX + myY * myY);
		return new Vec(myY / magnitude, myX / magnitude);
	}
	
	public Vec getOrthogonal()
	{
		return new Vec(-myX, myY);
	}
	
	public String toString()
	{
		return "(" + myX + "," + myY + ")";
	}
	
	public double getY()
	{
		return myY;
	}
	
	public double getX()
	{
		return myX;
	}

}
